#include <v_lib.h>
#include "rgcache.h"

typedef enum
{
	fg_constant = 1,
	fg_uniform = 2,
	fg_gaussian = 3,
	fg_iterators = 99,
	fg_linear_iterator = 100,
	fg_log10_iterator = 101
} float_generator_type;

typedef struct float_generator_s
{
	float_generator_type	type;
	char	*name;
	vboolean finished; /* iterations complete */
	union
	{
		vec_t constant;
		struct
		{
			vec_t min, max;
			rgcache_p rgc;
		} uniform;
		struct
		{
			vec_t mean, stdev;
			rgcache_p rgc;
		} gaussian;
		struct
		{
			u32_t steps, idx;
			vec_t from, to, val, delta;
		} ilinear;
		struct
		{
			u32_t steps, idx;
			vec_t log10from, log10to, val, log10delta;
		} ilog10;
	} u;
	vboolean (*next)(struct float_generator_s *self,vec_t *ret_val);
	void (*reset)(struct float_generator_s *self);
	void (*destroy)(struct float_generator_s *self);
} float_generator_t, *float_generator_p;

static vboolean
fg_next_constant(struct float_generator_s *self,vec_t *ret_val)
{
	*ret_val = self->u.constant;
	return true;
}

static void
fg_dummy_reset(struct float_generator_s *self)
{
	return;
}

static void
fg_dummy_destroy(struct float_generator_s *self)
{
	memFree(self->name);
	memFree(self);
	return;
}

vboolean
parseConstantGen(vhandle_t lex,vp_stack_p storage)
{
	float_generator_p fg = (float_generator_p)memAllocZ(sizeof(float_generator_t));
	fg->type = fg_constant;
	fg->name = strDup(lexNext(lex));
	fg->finished = false;
	fg->next = fg_next_constant;
	fg->reset = fg_dummy_reset;
	fg->destroy = fg_dummy_destroy;
	if(lexNextFloat(lex,&(fg->u.constant)))
	{
		vpsPush(storage,fg);
		return true;
	}
	memFree(fg->name);
	memFree(fg);
	return false;
}

vhandle_t in;

vp_stack_t floatgens = TYPED_STACK_EMPTY_INITIALIZER;

static void
clean_floatgens()
{
	float_generator_p g;
	while(vpsPop(&floatgens,(void_p*)&g))
		g->destroy(g);
	vpsClear(&floatgens);
} 

int main(int argc, char *argv[])
{
	CALLER_NAME(main);
	logStartConsoleLog();
	logUpperLevel(LOG_TRACE);
	in = lexParseFile("test.fcic","{}()[]");
	lexSetInteger(in,LEX_COMMENT_CHAR,'#');
	lexEnable(in,LEX_ONE_CHAR_COMMENTS);
	while(lexNext(in))
	{
		if(lexLastWas(in,"constant"))
		{
			if(!parseConstantGen(in, &floatgens))
				break;
		}
		else
		{
			PARSE_ERROR(in,"unknown keyword");
			break;
		}
	}
    printf("parsed %u generators\n", vpsCount(&floatgens));
    clean_floatgens();
	lexDestroy(in);
	memPrintStats();
	return 0;
}
